<?php
// $Id$

/**
 * @file
 * Contains helper functions to install CCK field definitions with.
 */

/**
 * Helper function, to create or update a content type given a definition.
 */
function _install_assist_update_type($type_values) {
  if (empty($type_values)) {
    drupal_set_message(t('The content type was not updated, no information was provided.'));
    return;
  }
  $field_data = $type_values['field_data'];
  unset($type_values['field_data']);
  node_types_rebuild();
  $type_values['has_title'] = ($type_values['title_label'] != '');
  $type_values['has_body '] = ($type_values['body_label'] != '');
  $new_content_type = node_type_set_defaults($type_values);
  variable_set('teaser_length_' . $new_content_type->type, 600);
  variable_set('node_preview_' . $new_content_type->type, 1);
  // Saving the content type after saving the variables allows modules to act
  // on those variables via hook_node_type_insert().
  $status = node_type_save($new_content_type);
  node_types_rebuild();
  if ($status == SAVED_UPDATED) {
    drupal_set_message(t('The content type %name has been updated.', array('%name' => $new_content_type->name)));
  }
  elseif ($status == SAVED_NEW) {
    drupal_set_message(t('The content type %name has been added.', array('%name' => $new_content_type->name)));
    watchdog('node', 'Added content type %name.', array('%name' => $new_content_type->name), WATCHDOG_NOTICE, l(t('view'), 'admin/structure/types'));
  }
  foreach ($field_data as $field_values) {
    _install_assist_update_field($type_values['type'], $field_values);
  }
}

/**
 * Helper function, to create or update a field and field instances per type given a definition.
 */
function _install_assist_update_field($type, $field_values) {
  if (empty($field_values)) {
    drupal_set_message(t('The content field was not updated, no field information was provided.'));
    return;
  }
  if (empty($type)) {
    drupal_set_message(t('The content field was not updated, no type was provided.'));
    return;
  }
  $field_values['entity_types'] = array('node');
  // Only one instance allowed per field. Avoiding entangling field instances.
  $instance_values = $field_values['instance_data'];
  if (empty($instance_values)) {
    drupal_set_message(t('The content field was not updated, no field instance information was provided.'));
    return;
  }
  $instance_values['field_name'] = $field_values['field_name'];
  $instance_values['entity_type'] = 'node';
  $instance_values['bundle'] = $type;
  unset($field_values['instance_data']);
  $field = field_info_field($field_values['field_name']);
  if (empty($field)) {
    $field = field_create_field($field_values);
    drupal_set_message(t('The content field %name has been added.', array('%name' => $field_values['field_name'])));
  }
  else {
    $field_source = field_info_field($field_values['field_name']);
    $field = array_merge($field_source, $field_values);
    field_update_field($field);
    drupal_set_message(t('The content field %name has been updated.', array('%name' => $field_values['field_name'])));
  }
  $field = field_info_field($field_values['field_name']);
  if (!empty($field)) {
    $instance = field_info_instance('node', $field_values['field_name'], $type);
    if (empty($instance)) {
      $instance = field_create_instance($instance_values);
      drupal_set_message(t('The content field %name has been added to %type.', array('%name' => $field_values['field_name'], '%type' => $type)));
    }
    else {
      $instance_source = field_read_instance('node', $instance['field_name'], $type);
      $instance = array_merge($instance_source, $instance_values);
      field_update_instance($instance);
      drupal_set_message(t('The content field %name has been updated for %type.', array('%name' => $field_values['field_name'], '%type' => $type)));
    }
  }
}

/**
 * Helper function, to create or update a content type given a definition.
 */
function _install_assist_delete_type($type_values) {
  if (empty($type_values)) {
    drupal_set_message(t('The content type was not deleted, no information was provided.'));
    return;
  }
  $field_data = $type_values['field_data'];
  foreach ($field_data as $field_values) {
    $include_additional = array(
      'include_inactive' => TRUE,
    );
    $instance = field_read_instance('node', $field_values['field_name'], $type_values['type'], $include_additional);
// If your widget is defined in the same module being uninstalled then the widget data won't come up on field_info_instance.
// And you won't be able to properly delete the field instance or field. Hence working around the reliance on "info" functions in delete functions.
// So first - we turn the widget and instances back on for a split second then we delete them.
    db_update('field_config_instance')
      ->fields(array('deleted' => 0))
      ->condition('field_name', $instance['field_name'])
      ->condition('entity_type', $instance['entity_type'])
      ->condition('bundle', $instance['bundle'])
      ->execute();
    db_update('field_config')
      ->fields(array('active' => 1))
      ->fields(array('storage_active' => 1))
      ->fields(array('deleted' => 0))
      ->condition('field_name', $instance['field_name'])
      ->execute();
    field_delete_instance($instance);
    field_delete_field($field_values['field_name']);
    drupal_set_message(t('The content field %name has been deleted, along with all instances.', array('%name' => $field_values['field_name'])));
  }
  node_type_delete($type_values['type']);
  node_types_rebuild();
  drupal_set_message(t('The content type %name has been deleted.', array('%name' => $type_values['type'])));
  watchdog('node', 'Deleted content type %name.', array('%name' => $type_values['type']), WATCHDOG_NOTICE, l(t('view'), 'admin/structure/types'));
}
